#!/usr/bin/env python
from gimpfu import *
import decimal

'''
###################
# EDGEY BLACK ver #
################### 
  Filter requires FX FOUNDRY's convlution matrix plugin.
  Filter will make subtle shading effects on things like
  flower petals at very low settings (.01 - 2.0 or so depends on size)
  A pixel shifter is included and will shift Horizontal and/or Vertical
  
  This release version is with hope of critique and or suggestion.
  .
  Brian Andersen December 17, 2016
  
  ...................................................... 
 COPYRIGHT NOTICE
###############################################################################
# This program is free software you can redistribute it and/or modify         #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation either version 3 of the License, or            #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY without even the implied warranty of               #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program if not, you can view the GNU General Public         #
# License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html  #
# Alternatively you can write to the Free Software Foundation, Inc., 675 Mass #
# Ave, Cambridge, MA 02139, USA.                                              #
###############################################################################
'''


def BA_EdgeyBlack(inImage, inDrawable, widthheight, ichoice, edgeamt, 
                  outpct, edgeang, invflag, shiftflag, hamt, vamt, ):

    pdb.gimp_context_push
    
    Hamt    = int(hamt)
    Vamt    = int(vamt) 
    IChoice = int(ichoice)

    Parms = 'C'+str(IChoice)+','+str(edgeamt)+','+str(outpct)+','    
    Invt = 'N,'
    if invflag:
        Invt = 'Y,'
    Shift='N'
    if shiftflag:
        Shift='S,'+str(Hamt)+','+str(Vamt)    

    name = inDrawable.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
        
    suffix = '_BA_Edgey,'+Parms+Invt+Shift

    width = inImage.width
    height = inImage.height
    type = inImage.base_type
    
    # calculate scale factor for the new output image
    if widthheight == 0 :   # means the user wants to have same size as input
        scale = 1
    else :     
        optiWidthHeight = widthheight       # size to work on to get best results
        realWidthHeight = width+height      # real size of the input image
        scale = decimal.Decimal(optiWidthHeight)/decimal.Decimal(realWidthHeight)   

    newwidth= int(width*scale)
    newheight=int(height*scale)

    newImage = pdb.gimp_image_new (newwidth, newheight, type)  
    
    oriBG = pdb.gimp_layer_new_from_drawable(inDrawable, newImage)        
    oriBG.mode = LAYER_MODE_NORMAL_LEGACY
    oriBG.name = name+suffix
    oriBG.opacity = 100.0 
    newImage.add_layer (oriBG, 0)
    pdb.gimp_item_set_visible(oriBG, 1) 
    pdb.gimp_layer_scale(oriBG, newwidth, newheight, False)
    alpha=pdb.gimp_drawable_has_alpha(oriBG)
    if (alpha==False):
        pdb.gimp_layer_add_alpha (oriBG)    #  RGBA    
   
    convLay = oriBG.copy()
    convLay.mode = LAYER_MODE_NORMAL_LEGACY
    convLay.name = "convLay"
    convLay.opacity = 100.0 
    newImage.add_layer(convLay, 0)    
    pdb.gimp_item_set_visible(convLay, 1)

    tmp2Lay = oriBG.copy()
    tmp2Lay.mode = LAYER_MODE_NORMAL_LEGACY
    tmp2Lay.name = "tmp2Lay"
    tmp2Lay.opacity = 100.0 
    newImage.add_layer(tmp2Lay, 0)    
    pdb.gimp_item_set_visible(tmp2Lay, 1)  

    if shiftflag:
        pdb.plug_in_shift(newImage, tmp2Lay, Hamt, 1)           
        pdb.plug_in_shift(newImage, tmp2Lay, Vamt, 0)        
    convLay = pdb.gimp_image_merge_down(newImage, tmp2Lay, 0)  # convLay name is convluted

    pdb.gimp_desaturate_full(convLay, 0)
    pdb.gimp_levels_stretch (convLay)    
 
    pdb.script_fu_convolution_presets_edge_detect_compass(newImage, convLay, edgeamt, edgeang,0,1,1,1,1,1,1,1,1)
    invtconvLay = convLay.copy()
    invtconvLay.name = 'E='+str(edgeamt)+' Out='+str(outpct)
    newImage.add_layer(invtconvLay, 0) 
    pdb.gimp_invert(invtconvLay)
    
    if invflag:
        pdb.gimp_invert(invtconvLay)
        pdb.gimp_invert(convLay)
    
    # Above is mode choice 0. Builds only the 3 layers at normal 100 visible

    if IChoice == 1:
        invtconvLay.name = 'E='+str(edgeamt)+' Out='+str(outpct)
        pdb.gimp_item_set_visible(invtconvLay,0)
        convLay.mode = LAYER_MODE_MULTIPLY_LEGACY
        convLay.opacity = outpct
        pdb.gimp_item_set_visible(convLay, 1)
        effect1 = pdb.gimp_layer_new_from_visible(newImage,newImage,'effect1')
        newImage.add_layer(effect1, 0)
        effect1.name = 'effect1'
        pdb.gimp_image_set_active_layer(newImage, effect1)
        pdb.gimp_item_set_visible(convLay, 0)
        pdb.gimp_item_set_visible(oriBG, 1)
        pdb.gimp_item_set_visible(effect1,1)

    elif IChoice == 2: 
        pdb.gimp_image_lower_item(newImage, convLay)
        oriBG1 = oriBG.copy()
        oriBG1.mode = LAYER_MODE_GRAIN_MERGE_LEGACY
        oriBG1.name = "OriBG1"
        oriBG1.opacity = outpct 
        newImage.add_layer(oriBG1, 0)  ##
        pdb.gimp_item_set_visible(oriBG1, 1)
        oriBG2 = oriBG.copy()
        oriBG2.mode = LAYER_MODE_GRAIN_MERGE_LEGACY
        oriBG2.name = "OriBG2"
        oriBG2.opacity = outpct 
        newImage.add_layer(oriBG2, 0)  ##
        pdb.gimp_item_set_visible(oriBG2, 1)
        invtconvLay.mode = LAYER_MODE_HARDLIGHT_LEGACY
        invtconvLay.opacity = outpct 
        invtconvLay.name = 'E='+str(edgeamt)+' Out='+str(outpct)
        oriBG.mode = LAYER_MODE_OVERLAY_LEGACY
        effect2 = pdb.gimp_layer_new_from_visible(newImage,newImage,'effect2')
        newImage.add_layer(effect2, 0)
        effect2.name = 'effect2' 
        pdb.gimp_item_set_visible(oriBG1,0)
        pdb.gimp_item_set_visible(oriBG2,0)
        pdb.gimp_item_set_visible(invtconvLay,0)
        pdb.gimp_item_set_visible(convLay,0) 
                
    if IChoice == 3:
        invtconvLay.mode = LAYER_MODE_BURN_LEGACY
        invtconvLay.opacity = outpct
        invtconvLay.name = 'E='+str(edgeamt)+' Out='+str(outpct)
        convLay.opacity = outpct 
        convLay.mode = LAYER_MODE_HSV_HUE_LEGACY
        oriBG.opacity = 100.0
        effect3 = pdb.gimp_layer_new_from_visible(newImage,newImage,'effect3')
        newImage.add_layer(effect3, 0)
        effect3.name = 'effect3'  
        pdb.gimp_item_set_visible(invtconvLay,0)
        pdb.gimp_item_set_visible(convLay,0)        
        
    # Clean Up
    pdb.gimp_display_new (newImage)
    pdb.gimp_displays_flush
   
    # Close the undo group.
    pdb.gimp_context_pop

    
    return(BA_EdgeyBlack)             

# This is the plugin registration function
register(
    "BA_EdgeyBlack",   
    "BA_EDGEY effects {NEEDS FX FOUNDRY convlution matrix}",   
    "Black version in hopes of critique/suggestion",
    "Brian Andersen", 
    "animicule@gimpchat.com", 
    "December 16, 2016",
    "BA Edgey Black...", 
    "RGB*", 

    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of New Image (0=same as input)", 0, (0, 9600, 300)),
      (PF_OPTION, "ichoice", "Choose Style of Outcome ", 1, ("Base Layers", "IvtMult", "GM_SL", "Burn",)),
      (PF_ADJUSTMENT, "edgeamt", "Effect Edge Amount .05-.29 subtle 20 for extreme", .2, (0.01,100,.01)),
      (PF_ADJUSTMENT, "outpct", "Effect Outcome Strength Percent ", 10.0, (1.0,100.0,1.0)),      
      (PF_ADJUSTMENT, "edgeang", "Azimuth in 45 degree increments 0-7", 0, (0,7,1)),
      (PF_TOGGLE, "invflag"  , "Invert Convolution (black to white)?"  , False),
      (PF_TOGGLE, "shiftflag", "Shift pixels lightly for a hairy edge?", False),
      (PF_ADJUSTMENT, "Hamt", "How many pixels Horizontal", 2, (0,100,1)),
      (PF_ADJUSTMENT, "Vamt", "How many pixels Vertical  ", 1, (0,100,1)),     

    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    BA_EdgeyBlack,
    menu="<Image>/Filters/Artistic/Brian/",
    )

main()
